#!/usr/bin/env python

"""
A WebStack application for "playing" questionnaires designed by the
questionnaire editor.
"""

import WebStack.Generic
from XSLForms.Resources.WebResources import \
    XSLFormsResource, input, output, resources, prepare_resources as xslforms_prepare_resources
import os
import libxml2dom
from WebStack.Repositories.Directory import DirectoryRepository
import time

# Site map imports.

from WebStack.Resources.ResourceMap import MapResource
from WebStack.Resources.Selectors import EncodingSelector
from WebStack.Resources.Static import DirectoryResource

# Configuration settings.

encoding = "utf-8"
storage_directory = os.path.join(os.getcwd(), "Responses")

# Resource classes.

class QuestionnairePlayerResource(XSLFormsResource):

    "A resource providing a questionnaire editor."

    resource_dir = resources(__file__)
    template_resources = {
        "questionnaire" : output("player_template.xhtml")
        }
    init_resources = {
        "questionnaire" : input("player_template.xhtml")
        }
    document_resources = {
        "questions" : "questions.xml"
        }
    transform_resources = {
        "distill" : ["distill_responses.xsl"]
        }

    def select_activity(self, trans, form):
        form.set_activity("questionnaire")

    def create_document(self, trans, form):
        if XSLFormsResource.create_document(self, trans, form):
            questions_xml = self.prepare_document("questions")
            questions = libxml2dom.parse(questions_xml)
            doc = form.get_document()
            imported_questions = doc.importNode(questions.documentElement, 1)
            doc.replaceChild(imported_questions, doc.documentElement)

    def respond_to_input(self, trans, form):

        parameters = form.get_parameters()
        selectors = form.get_selectors()
        questionnaire = form.get_document()

        # Send a response according to certain parameters.
        # When saved, an XML version of the data is employed.

        if parameters.has_key("finish"):
            repository = DirectoryRepository(storage_directory)
            key = str(time.time())
            output_filename = repository.lock(key, 1)
            distilled_output_filename = repository.lock("distilled-" + key, 1)
            try:
                distill_xsl = self.prepare_transform("distill")
                distilled = self.get_result(distill_xsl, questionnaire)
                distilled.toStream(open(distilled_output_filename, "wb"), "utf-8", prettyprint=1)
                questionnaire.toStream(open(output_filename, "wb"), "utf-8", prettyprint=1)
                questionnaire.documentElement.setAttribute("submitted", "true")
            finally:
                repository.unlock(key)
                repository.unlock("distilled-" + key)

# Site map initialisation.

def get_site():

    "Return a simple Web site resource."

    # Get the main resource and the directory used by the application.

    questionnaire_resource = QuestionnairePlayerResource()
    directory = questionnaire_resource.resource_dir

    # Make a simple Web site.

    resource = MapResource({
        # Static resources:
        "styles" : DirectoryResource(os.path.join(directory, "styles"), {"css" : "text/css"}),
        # Main page:
        "" : questionnaire_resource
        })

    return EncodingSelector(resource, encoding)

# Resource preparation ahead of time - useful for making installations.

def prepare_resources():
    for cls in [QuestionnairePlayerResource]:
        xslforms_prepare_resources(cls)

# vim: tabstop=4 expandtab shiftwidth=4
